require 'haml'
require 'premailer'
require 'nokogiri'
require 'sassc'

class ChannelResearchStationery
  def initialize(content, options = {})
    @content = content
    @primary_color = options.delete(:primary_color) || '#6d6e71'
    @options = default_options.merge(options)
  end

  def to_text
    Premailer.new(to_html,
      {
        warn_level: Premailer::Warnings::SAFE
      }
    ).to_plain_text
  end

  def to_html
    template_and_content
  end

  def template_and_content
    template = Nokogiri::HTML(template_with_inline_css)    
    content =  Nokogiri::HTML(content_with_inline_css)
    template.at('style').remove
    template.at('div#content') << content.at('body')
    # Add content-editable bits
    template.xpath('//*[@class="contenteditable"]').each do |node|
      node['contenteditable'] = 'true'
    end
    return template
  end

  def template_with_inline_css
    Premailer.new(template,
      {
        warn_level: Premailer::Warnings::SAFE,
        css_string: template_css_string,
        with_html_string: true
      }
    ).to_inline_css
  end

  def content_with_inline_css
    Premailer.new(@content,
      {
        warn_level: Premailer::Warnings::SAFE,
        css_string: content_css_string,
        with_html_string: true
      }
    ).to_inline_css
  end

  def placeholder_content
    contents = File.open("lib/generators/templates/placeholder_contents/#{@options['placeholder']}").read
    Haml::Engine.new(contents).render
  end

  def template
    template_path = File.join( __dir__, "./generators/templates/template.html.haml" )
    contents = File.open(template_path).read
    engine = Haml::Engine.new(contents)
    engine.render(Object.new, { options: @options })
  end

  def template_css_string
    reset_css + template_styles_css
  end

  def content_css_string
    reset_css + content_styles_css + @options[:extra_css]
  end

  def reset_css
    css_path = File.join( __dir__, "./generators/templates/reset.css" )
    File.open(css_path).read
  end

  def sass_options
    @options.select {|k,v| [:background_color,:header_background_color,:width,:cta_background_color,:cta_color,:bold_color,:link_color].include?(k)}
  end

  def sass_variables
    sass_options.map {|k,v| "$#{k}: #{v};"}.join("\n")
  end

  def template_styles_css
    css_path = File.join( __dir__, "./generators/templates/template_styles.sass" )
    sass_styles = File.open(css_path, "rb").read
    sass = sass_variables + "\n" + sass_styles
    SassC::Engine.new(sass, style: :compressed).render
  end

  def content_styles_css
    css_path = File.join( __dir__, "./generators/templates/content_styles.sass" )
    sass_styles = File.open(css_path, "rb").read
    sass = sass_variables + "\n" + sass_styles
    SassC::Engine.new(sass, style: :compressed).render
  end


  def default_options
    {
      header_background_color: @primary_color,
      link_color: @primary_color,
      bold_color: @primary_color,
      cta_background_color: @primary_color,
      background_color: '#ffffff',
      cta_color: '#ffffff',
      width: '700px',
      badge_left: false,
      badge_right: false,
      title: false,
      subtitle: false,
      extra_css: ''
    }
  end
end